{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Core.Classes.SdqpSolver
       (
         SdqpSolver,
         SdqpSolverClass(..),
         sdqpSolver,
         sdqpSolver_doc,
         sdqpSolver_hasPlugin,
         sdqpSolver_loadPlugin,
         sdqpSolver_setSOCQPOptions,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Marshal ( new, free )
import Foreign.Storable ( peek )
import Foreign.Ptr ( Ptr, nullPtr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Internal.CToolsInstances ( )
import Casadi.Internal.FormatException ( formatException )
import Casadi.Internal.MarshalTypes ( StdVec, StdString) -- StdPair StdOstream'
import Casadi.Internal.Marshal ( Marshal(..), withMarshal )
import Casadi.Internal.WrapReturn ( WrapReturn(..) )
import Casadi.Core.Data
import Casadi.Core.Enums
-- direct wrapper
foreign import ccall unsafe "casadi__SdqpSolver__CONSTRUCTOR" c_casadi__SdqpSolver__CONSTRUCTOR
  :: Ptr (Ptr StdString) -> IO (Ptr SdqpSolver')
casadi__SdqpSolver__CONSTRUCTOR
  :: IO SdqpSolver
casadi__SdqpSolver__CONSTRUCTOR  =

  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SdqpSolver__CONSTRUCTOR errStrPtrP 
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
sdqpSolver :: IO SdqpSolver
sdqpSolver = casadi__SdqpSolver__CONSTRUCTOR


-- direct wrapper
foreign import ccall unsafe "casadi__SdqpSolver__doc" c_casadi__SdqpSolver__doc
  :: Ptr (Ptr StdString) -> Ptr StdString -> IO (Ptr StdString)
casadi__SdqpSolver__doc
  :: String -> IO String
casadi__SdqpSolver__doc x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SdqpSolver__doc errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
sdqpSolver_doc :: String -> IO String
sdqpSolver_doc = casadi__SdqpSolver__doc


-- direct wrapper
foreign import ccall unsafe "casadi__SdqpSolver__hasPlugin" c_casadi__SdqpSolver__hasPlugin
  :: Ptr (Ptr StdString) -> Ptr StdString -> IO CInt
casadi__SdqpSolver__hasPlugin
  :: String -> IO Bool
casadi__SdqpSolver__hasPlugin x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SdqpSolver__hasPlugin errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
sdqpSolver_hasPlugin :: String -> IO Bool
sdqpSolver_hasPlugin = casadi__SdqpSolver__hasPlugin


-- direct wrapper
foreign import ccall unsafe "casadi__SdqpSolver__loadPlugin" c_casadi__SdqpSolver__loadPlugin
  :: Ptr (Ptr StdString) -> Ptr StdString -> IO ()
casadi__SdqpSolver__loadPlugin
  :: String -> IO ()
casadi__SdqpSolver__loadPlugin x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SdqpSolver__loadPlugin errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
sdqpSolver_loadPlugin :: String -> IO ()
sdqpSolver_loadPlugin = casadi__SdqpSolver__loadPlugin


-- direct wrapper
foreign import ccall unsafe "casadi__SdqpSolver__setSOCQPOptions" c_casadi__SdqpSolver__setSOCQPOptions
  :: Ptr (Ptr StdString) -> Ptr SdqpSolver' -> IO ()
casadi__SdqpSolver__setSOCQPOptions
  :: SdqpSolver -> IO ()
casadi__SdqpSolver__setSOCQPOptions x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SdqpSolver__setSOCQPOptions errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
sdqpSolver_setSOCQPOptions :: SdqpSolverClass a => a -> IO ()
sdqpSolver_setSOCQPOptions x = casadi__SdqpSolver__setSOCQPOptions (castSdqpSolver x)

