{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Core.Classes.SymbolicNLP
       (
         SymbolicNLP,
         SymbolicNLPClass(..),
         symbolicNLP_getDescription,
         symbolicNLP_getRepresentation,
         symbolicNLP_parseNL,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Marshal ( new, free )
import Foreign.Storable ( peek )
import Foreign.Ptr ( Ptr, nullPtr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Internal.CToolsInstances ( )
import Casadi.Internal.FormatException ( formatException )
import Casadi.Internal.MarshalTypes ( StdVec, StdString) -- StdPair StdOstream'
import Casadi.Internal.Marshal ( Marshal(..), withMarshal )
import Casadi.Internal.WrapReturn ( WrapReturn(..) )
import Casadi.Core.Data
import Casadi.Core.Enums
-- direct wrapper
foreign import ccall unsafe "casadi__SymbolicNLP__parseNL" c_casadi__SymbolicNLP__parseNL
  :: Ptr (Ptr StdString) -> Ptr SymbolicNLP' -> Ptr StdString -> IO ()
casadi__SymbolicNLP__parseNL
  :: SymbolicNLP -> String -> IO ()
casadi__SymbolicNLP__parseNL x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SymbolicNLP__parseNL errStrPtrP x0' x1'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
symbolicNLP_parseNL :: SymbolicNLPClass a => a -> String -> IO ()
symbolicNLP_parseNL x = casadi__SymbolicNLP__parseNL (castSymbolicNLP x)


-- direct wrapper
foreign import ccall unsafe "casadi__SymbolicNLP__getRepresentation" c_casadi__SymbolicNLP__getRepresentation
  :: Ptr (Ptr StdString) -> Ptr SymbolicNLP' -> IO (Ptr StdString)
casadi__SymbolicNLP__getRepresentation
  :: SymbolicNLP -> IO String
casadi__SymbolicNLP__getRepresentation x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SymbolicNLP__getRepresentation errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
symbolicNLP_getRepresentation :: SymbolicNLPClass a => a -> IO String
symbolicNLP_getRepresentation x = casadi__SymbolicNLP__getRepresentation (castSymbolicNLP x)


-- direct wrapper
foreign import ccall unsafe "casadi__SymbolicNLP__getDescription" c_casadi__SymbolicNLP__getDescription
  :: Ptr (Ptr StdString) -> Ptr SymbolicNLP' -> IO (Ptr StdString)
casadi__SymbolicNLP__getDescription
  :: SymbolicNLP -> IO String
casadi__SymbolicNLP__getDescription x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SymbolicNLP__getDescription errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
symbolicNLP_getDescription :: SymbolicNLPClass a => a -> IO String
symbolicNLP_getDescription x = casadi__SymbolicNLP__getDescription (castSymbolicNLP x)

